package com.hai.openApi.service;

import com.alibaba.fastjson.JSONObject;
import com.hai.entity.ApiOrder;
import com.hai.entity.HighChildOrder;
import com.hai.entity.HighDiscountUserRel;
import com.hai.entity.HighOrder;
import com.hai.order.model.CreateOrderChildModel;
import com.hai.order.model.CreateOrderModel;

import java.util.List;

/**
 * 创建订单-业务校验、组装数据
 * @className: CreateOrderCheckService
 * @author: HuRui
 * @date: 2022/8/25
 **/
public interface ApiOrderCreateHandleService {

    /**
     * @Author Sum1Dream
     * @Name oilHandle
     * @Description // 在线加油校验操作
     * @Date 13:59 2023/3/1
     * @Param [object]
     * @Return com.hai.entity.ApiOrder
     */
    ApiOrder oilHandle(JSONObject object) throws Exception;

    /**
     * @Author Sum1Dream
     * @Name mobileHandle
     * @Description // 话费校验操作
     * @Date 15:38 2023/3/13
     * @Param [object]
     * @Return com.hai.entity.ApiOrder
     */
    ApiOrder mobileHandle(JSONObject object) throws Exception;

    /**
     * @Author Sum1Dream
     * @Name memberRechargeHandle
     * @Description // 第三方会员充值校验操作
     * @Date 14:42 2023/3/15
     * @Param [object]
     * @Return com.hai.entity.ApiOrder
     */
    ApiOrder memberRechargeHandle(JSONObject object) throws Exception;

    /**
     * @Author Sum1Dream
     * @Name kfcHandle
     * @Description //TODO
     * @Date 14:44 2023/3/15
     * @Param [object]
     * @Return com.hai.entity.ApiOrder
     */
    ApiOrder kfcHandle(JSONObject object) throws Exception;

    /**
     * @Author Sum1Dream
     * @Name starbucksHandle
     * @Description // 星巴克操作
     * @Date 17:58 2023/4/4
     * @Param [object]
     * @Return com.hai.entity.ApiOrder
     */
    ApiOrder starbucksHandle(JSONObject object) throws Exception;

    ApiOrder blxCouponHandle(JSONObject object) throws Exception;

}
